package kaka.cakelight;

public class Color {
    private int r, g, b;

    public static Color rgb(double r, double g, double b) {
        return rgb((int)(255 * r), (int)(255 * g), (int)(255 * b));
    }

    public static Color rgb(int r, int g, int b) {
        Color c = new Color();
        c.r = r;
        c.g = g;
        c.b = b;
        return c;
    }

    public static Color hsv(double hue, double saturation, double value) {
        double normalizedHue = hue - Math.floor(hue);
        int h = (int)(normalizedHue * 6);
        double f = normalizedHue * 6 - h;
        double p = value * (1 - saturation);
        double q = value * (1 - f * saturation);
        double t = value * (1 - (1 - f) * saturation);

        switch (h) {
            case 0: return rgb(value, t, p);
            case 1: return rgb(q, value, p);
            case 2: return rgb(p, value, t);
            case 3: return rgb(p, q, value);
            case 4: return rgb(t, p, value);
            case 5: return rgb(value, p, q);
            default: throw new RuntimeException("Something went wrong when converting from HSV to RGB. Input was " + hue + ", " + saturation + ", " + value);
        }
    }

    public int r() {
        return r;
    }

    public int g() {
        return g;
    }

    public int b() {
        return b;
    }

    public Color interpolate(Color other, double value) {
        double invertedValue = 1 - value;
        return Color.rgb(
                (int)(r * invertedValue + other.r * value),
                (int)(g * invertedValue + other.g * value),
                (int)(b * invertedValue + other.b * value)
        );
    }
}
