package kaka.cakelight;

public class LedFrame {
    private Configuration config;
    private byte[] bytes;
    private int roff = 0, goff = 2, boff = 1; // Color values are stored as RBG, which is what the LED list takes.

    public static LedFrame from(Configuration config) {
        LedFrame frame = new LedFrame();
        frame.config = config;
        frame.bytes = new byte[config.leds.getCount() * 3];
        return frame;
    }

    public void fillColor(int r, int g, int b) {
        for (int i = 0; i < bytes.length; i += 3) {
            bytes[i + roff] = (byte)r;
            bytes[i + goff] = (byte)g;
            bytes[i + boff] = (byte)b;
        }
    }

    public void fillColor(Color color) {
        fillColor(color.r(), color.g(), color.b());
    }

    public void setLedColor(int led, Color color) {
        int offset = led * 3;
        bytes[offset + roff] = (byte)color.r();
        bytes[offset + goff] = (byte)color.g();
        bytes[offset + boff] = (byte)color.b();
    }

    public Color getLedColor(int led) {
        int offset = led * 3;
        return Color.rgb(
                bytes[offset + roff] & 0xff,
                bytes[offset + goff] & 0xff,
                bytes[offset + boff] & 0xff
        );
    }

    public byte[] getBytes() {
        return bytes;
    }

    // TODO this needs to be improved
    /** The x position of the led from 0.0-1.0. */
    public double xOf(int led) {
        /* left   */ if (led >= config.leds.cols * 2 + config.leds.rows) return 0;
        /* top    */ if (led >= config.leds.cols + config.leds.rows) return 1 - (double)(led - config.leds.cols - config.leds.rows) / config.leds.cols;
        /* right  */ if (led >= config.leds.cols) return 1;
        /* bottom */ return (double)led / config.leds.cols;
    }

    /** The y position of the led from 0.0-1.0. */
    public double yOf(int led) {
        /* left   */ if (led >= config.leds.cols * 2 + config.leds.rows) return (double)(led - config.leds.cols * 2 - config.leds.rows) / config.leds.rows;
        /* top    */ if (led >= config.leds.cols + config.leds.rows) return 0;
        /* right  */ if (led >= config.leds.cols) return 1 - (double)(led - config.leds.cols) / config.leds.rows;
        /* bottom */ return 1;
    }
}
